==> grib-api_1.7.0.orig.tar.gz: grib_api-1.7.0/src/grib_accessor_class_g1day_of_the_year_date.c <==
/**
* Copyright 2005-2007 ECMWF
* 
* Licensed under the GNU Lesser General Public License which
* incorporates the terms and conditions of version 3 of the GNU
* General Public License.
* See LICENSE and gpl-3.0.txt for details.
*/

#include "grib_api_internal.h"

/* 
   This is used by make_class.pl

   START_CLASS_DEF
   CLASS      = accessor
   SUPER      = grib_accessor_class_g1date
   IMPLEMENTS = unpack_string
   IMPLEMENTS = dump
   IMPLEMENTS = init
   END_CLASS_DEF

 */

/* START_CLASS_IMP */

/*

Don't edit anything between START_CLASS_IMP and END_CLASS_IMP
Instead edit values between START_CLASS_DEF and END_CLASS_DEF
or edit "accessor.class" and rerun ./make_class.pl

*/

static int unpack_string (grib_accessor*, char*, size_t *len);
static void dump(grib_accessor*, grib_dumper*);
static void init(grib_accessor*,const long, grib_arguments* );
static void init_class(grib_accessor_class*);

typedef struct grib_accessor_g1day_of_the_year_date {
    grib_accessor          att;
/* Members defined in gen */
/* Members defined in long */
/* Members defined in g1date */
	const char* century;
	const char* year;
	const char* month;
	const char* day;
/* Members defined in g1day_of_the_year_date */
} grib_accessor_g1day_of_the_year_date;

extern grib_accessor_class* grib_accessor_class_g1date;

static grib_accessor_class _grib_accessor_class_g1day_of_the_year_date = {
    &grib_accessor_class_g1date,                      /* super                     */
    "g1day_of_the_year_date",                      /* name                      */
    sizeof(grib_accessor_g1day_of_the_year_date),  /* size                      */
    0,                           /* inited */
    &init_class,                 /* init_class */
    &init,                       /* init                      */
    0,                  /* post_init                      */
    0,                    /* free mem                       */
    &dump,                       /* describes himself         */
    0,                /* get length of section     */
    0,                /* get number of values      */
    0,                 /* get number of bytes      */
    0,                /* get offset to bytes           */
    0,            /* get native type               */
    0,                /* get sub_section                */
    0,               /* grib_pack procedures long      */
    0,               /* grib_pack procedures long      */
    0,                  /* grib_pack procedures long      */
    0,                /* grib_unpack procedures long    */
    0,                /* grib_pack procedures double    */
    0,              /* grib_unpack procedures double  */
    0,                /* grib_pack procedures string    */
    &unpack_string,              /* grib_unpack procedures string  */
    0,                 /* grib_pack procedures bytes     */
    0,               /* grib_unpack procedures bytes   */
    0,            /* pack_expression */
    0,              /* notify_change   */
    0,                /* update_size   */
    0,            /* preferred_size   */
    0,                    /* resize   */
    0,      /* nearest_smaller_value */
    0,                       /* next accessor    */
    0,                    /* compare vs. another accessor   */
    0,             /* unpack only ith value          */
};


grib_accessor_class* grib_accessor_class_g1day_of_the_year_date = &_grib_accessor_class_g1day_of_the_year_date;


static void init_class(grib_accessor_class* c)
{
	c->next_offset	=	(*(c->super))->next_offset;
	c->value_count	=	(*(c->super))->value_count;
	c->byte_count	=	(*(c->super))->byte_count;
	c->byte_offset	=	(*(c->super))->byte_offset;
	c->get_native_type	=	(*(c->super))->get_native_type;
	c->sub_section	=	(*(c->super))->sub_section;
	c->pack_missing	=	(*(c->super))->pack_missing;
	c->is_missing	=	(*(c->super))->is_missing;
	c->pack_long	=	(*(c->super))->pack_long;
	c->unpack_long	=	(*(c->super))->unpack_long;
	c->pack_double	=	(*(c->super))->pack_double;
	c->unpack_double	=	(*(c->super))->unpack_double;
	c->pack_string	=	(*(c->super))->pack_string;
	c->pack_bytes	=	(*(c->super))->pack_bytes;
	c->unpack_bytes	=	(*(c->super))->unpack_bytes;
	c->pack_expression	=	(*(c->super))->pack_expression;
	c->notify_change	=	(*(c->super))->notify_change;
	c->update_size	=	(*(c->super))->update_size;
	c->preferred_size	=	(*(c->super))->preferred_size;
	c->resize	=	(*(c->super))->resize;
	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
	c->next	=	(*(c->super))->next;
	c->compare	=	(*(c->super))->compare;
	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
}
/* END_CLASS_IMP */


static void init(grib_accessor* a,const long l, grib_arguments* c)
{
	a->flags |= GRIB_ACCESSOR_FLAG_READ_ONLY;
}


static void dump(grib_accessor* a, grib_dumper* dumper)
{
	grib_dump_string(dumper,a,NULL);
}

static int unpack_string(grib_accessor* a, char* val, size_t *len)
{   
	/* special clim case where each mont have 30 days.. to comply with mars*/
	grib_accessor_g1day_of_the_year_date* self = (grib_accessor_g1day_of_the_year_date*)a;


	char tmp[1024];

	long year = 0;
	long century = 0;
	long month = 0;
	long day = 0;

	long fullyear = 0;
	long fake_day_of_year = 0; 


	size_t l;

	grib_get_long_internal(a->parent->h, self->century,&century);
	grib_get_long_internal(a->parent->h, self->day,&day);
	grib_get_long_internal(a->parent->h, self->month,&month);
	grib_get_long_internal(a->parent->h, self->year,&year);

	if(*len < 1)
		return GRIB_BUFFER_TOO_SMALL;


	fullyear = ((century-1)*100 + year);
	fake_day_of_year =  ((month-1) * 30) + day;
	sprintf(tmp,"%04ld-%03ld",fullyear,fake_day_of_year);

	l = strlen(tmp) + 1;
	if(*len < l)
	{
		*len = l;
		return GRIB_BUFFER_TOO_SMALL;
	}

	*len = l;
	strcpy(val,tmp);


	return GRIB_SUCCESS;
}

==> grib-api_1.8.0.orig.tar.gz: grib_api-1.8.0/src/grib_accessor_class_g1day_of_the_year_date.c <==
/**
* Copyright 2005-2007 ECMWF
* 
* Licensed under the GNU Lesser General Public License which
* incorporates the terms and conditions of version 3 of the GNU
* General Public License.
* See LICENSE and gpl-3.0.txt for details.
*/

#include "grib_api_internal.h"

/* 
   This is used by make_class.pl

   START_CLASS_DEF
   CLASS      = accessor
   SUPER      = grib_accessor_class_g1date
   IMPLEMENTS = unpack_string
   IMPLEMENTS = dump
   IMPLEMENTS = init
   END_CLASS_DEF

 */

/* START_CLASS_IMP */

/*

Don't edit anything between START_CLASS_IMP and END_CLASS_IMP
Instead edit values between START_CLASS_DEF and END_CLASS_DEF
or edit "accessor.class" and rerun ./make_class.pl

*/

static int unpack_string (grib_accessor*, char*, size_t *len);
static void dump(grib_accessor*, grib_dumper*);
static void init(grib_accessor*,const long, grib_arguments* );
static void init_class(grib_accessor_class*);

typedef struct grib_accessor_g1day_of_the_year_date {
    grib_accessor          att;
/* Members defined in gen */
/* Members defined in long */
/* Members defined in g1date */
	const char* century;
	const char* year;
	const char* month;
	const char* day;
/* Members defined in g1day_of_the_year_date */
} grib_accessor_g1day_of_the_year_date;

extern grib_accessor_class* grib_accessor_class_g1date;

static grib_accessor_class _grib_accessor_class_g1day_of_the_year_date = {
    &grib_accessor_class_g1date,                      /* super                     */
    "g1day_of_the_year_date",                      /* name                      */
    sizeof(grib_accessor_g1day_of_the_year_date),  /* size                      */
    0,                           /* inited */
    &init_class,                 /* init_class */
    &init,                       /* init                      */
    0,                  /* post_init                      */
    0,                    /* free mem                       */
    &dump,                       /* describes himself         */
    0,                /* get length of section     */
    0,                /* get number of values      */
    0,                 /* get number of bytes      */
    0,                /* get offset to bytes           */
    0,            /* get native type               */
    0,                /* get sub_section                */
    0,               /* grib_pack procedures long      */
    0,               /* grib_pack procedures long      */
    0,                  /* grib_pack procedures long      */
    0,                /* grib_unpack procedures long    */
    0,                /* grib_pack procedures double    */
    0,              /* grib_unpack procedures double  */
    0,                /* grib_pack procedures string    */
    &unpack_string,              /* grib_unpack procedures string  */
    0,                 /* grib_pack procedures bytes     */
    0,               /* grib_unpack procedures bytes   */
    0,            /* pack_expression */
    0,              /* notify_change   */
    0,                /* update_size   */
    0,            /* preferred_size   */
    0,                    /* resize   */
    0,      /* nearest_smaller_value */
    0,                       /* next accessor    */
    0,                    /* compare vs. another accessor   */
    0,     /* unpack only ith value          */
    0,             		/* clear          */
};


grib_accessor_class* grib_accessor_class_g1day_of_the_year_date = &_grib_accessor_class_g1day_of_the_year_date;


static void init_class(grib_accessor_class* c)
{
	c->next_offset	=	(*(c->super))->next_offset;
	c->value_count	=	(*(c->super))->value_count;
	c->byte_count	=	(*(c->super))->byte_count;
	c->byte_offset	=	(*(c->super))->byte_offset;
	c->get_native_type	=	(*(c->super))->get_native_type;
	c->sub_section	=	(*(c->super))->sub_section;
	c->pack_missing	=	(*(c->super))->pack_missing;
	c->is_missing	=	(*(c->super))->is_missing;
	c->pack_long	=	(*(c->super))->pack_long;
	c->unpack_long	=	(*(c->super))->unpack_long;
	c->pack_double	=	(*(c->super))->pack_double;
	c->unpack_double	=	(*(c->super))->unpack_double;
	c->pack_string	=	(*(c->super))->pack_string;
	c->pack_bytes	=	(*(c->super))->pack_bytes;
	c->unpack_bytes	=	(*(c->super))->unpack_bytes;
	c->pack_expression	=	(*(c->super))->pack_expression;
	c->notify_change	=	(*(c->super))->notify_change;
	c->update_size	=	(*(c->super))->update_size;
	c->preferred_size	=	(*(c->super))->preferred_size;
	c->resize	=	(*(c->super))->resize;
	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
	c->next	=	(*(c->super))->next;
	c->compare	=	(*(c->super))->compare;
	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
	c->clear	=	(*(c->super))->clear;
}
/* END_CLASS_IMP */


static void init(grib_accessor* a,const long l, grib_arguments* c)
{
	a->flags |= GRIB_ACCESSOR_FLAG_READ_ONLY;
}


static void dump(grib_accessor* a, grib_dumper* dumper)
{
	grib_dump_string(dumper,a,NULL);
}

static int unpack_string(grib_accessor* a, char* val, size_t *len)
{   
	/* special clim case where each mont have 30 days.. to comply with mars*/
	grib_accessor_g1day_of_the_year_date* self = (grib_accessor_g1day_of_the_year_date*)a;


	char tmp[1024];

	long year = 0;
	long century = 0;
	long month = 0;
	long day = 0;

	long fullyear = 0;
	long fake_day_of_year = 0; 


	size_t l;

	grib_get_long_internal(a->parent->h, self->century,&century);
	grib_get_long_internal(a->parent->h, self->day,&day);
	grib_get_long_internal(a->parent->h, self->month,&month);
	grib_get_long_internal(a->parent->h, self->year,&year);

	if(*len < 1)
		return GRIB_BUFFER_TOO_SMALL;


	fullyear = ((century-1)*100 + year);
	fake_day_of_year =  ((month-1) * 30) + day;
	sprintf(tmp,"%04ld-%03ld",fullyear,fake_day_of_year);

	l = strlen(tmp) + 1;
	if(*len < l)
	{
		*len = l;
		return GRIB_BUFFER_TOO_SMALL;
	}

	*len = l;
	strcpy(val,tmp);


	return GRIB_SUCCESS;
}
